@@ -6,15 +6,21 @@ Implementing a basic command-line interface.
 ## under GNU GPL v3 license
 
 import argparse
+import atexit
 import logging
 import sys
+import warnings
 
 from concurrent.futures import ThreadPoolExecutor, as_completed
+from platform import python_version
+from threading import Lock
 
+from . import __version__
 from .cli_utils import (load_blacklist, load_input_dict, load_input_urls,
-                        add_to_compressed_dict, cli_crawler,
+                        cli_crawler,
                         file_processing_pipeline, url_processing_pipeline,
                         examine, write_result)
+from .downloads import add_to_compressed_dict
 from .feeds import find_feed_urls
 from .settings import DOWNLOAD_THREADS
 from .sitemaps import sitemap_search
@@ -22,6 +28,8 @@ from .sitemaps import sitemap_search
 
 LOGGER = logging.getLogger(__name__)
 
+INPUTDICT = None
+THREAD_LOCK = Lock()
 
 # fix output encoding on some systems
 try:
@@ -92,6 +100,9 @@ def parse_args(args):
     group3_ex.add_argument("--crawl",
                         help="crawl a fixed number of pages within a website starting from the given URL",
                         nargs='?', const=True, default=False)
+    group3_ex.add_argument("--explore",
+                        help="explore the given websites (combination of sitemap and crawl)",
+                        nargs='?', const=True, default=False)
     group3.add_argument('--archived',
                         help='try to fetch URLs from the Internet Archive if downloads fail',
                         action="store_true")
@@ -115,17 +126,23 @@ def parse_args(args):
                         help="include image sources in output (experimental)",
                         action="store_true")
     group4.add_argument("--nocomments",
+                        help=argparse.SUPPRESS,
+                        action="store_false")  # will be deprecated
+    group4.add_argument("--notables",
+                        help=argparse.SUPPRESS,
+                        action="store_false")  # will be deprecated
+    group4.add_argument("--no-comments",
                         help="don't output any comments",
                         action="store_false")  # false = no comments
-    group4.add_argument("--notables",
+    group4.add_argument("--no-tables",
                         help="don't output any table elements",
                         action="store_false")  # false = no tables
     group4.add_argument("--only-with-metadata",
                         help="only output those documents with title, URL and date (for formats supporting metadata)",
                         action="store_true")
     group4.add_argument("--with-metadata",
-                        help="similar (will be deprecated)",
-                        action="store_true")
+                        help=argparse.SUPPRESS,
+                        action="store_true")   # will be deprecated
     group4.add_argument("--target-language",
                         help="select a target language (ISO 639-1 codes)",
                         type=str)
@@ -135,6 +152,12 @@ def parse_args(args):
     group4.add_argument("--config-file",
                         help="override standard extraction parameters with a custom config file",
                         type=str)
+    group4.add_argument("--precision",
+                        help="favor extraction precision (less noise, possibly less text)",
+                        action="store_true")
+    group4.add_argument("--recall",
+                        help="favor extraction recall (more text, possibly more noise)",
+                        action="store_true")
 
     # https://docs.python.org/3/library/argparse.html#argparse.ArgumentParser.add_mutually_exclusive_group
     group5_ex.add_argument('-out', '--output-format',
@@ -160,13 +183,21 @@ def parse_args(args):
     parser.add_argument('-v', '--verbose', action='count', default=0,
                         help="increase logging verbosity (-v or -vv)",
                         )
+    parser.add_argument(
+        "--version",
+        help="show version information and exit",
+        action="version",
+        version=f"Trafilatura {__version__} - Python {python_version()}",
+    )
+
 
     # wrap in mapping to prevent invalid input
     return map_args(parser.parse_args())
 
 
 def map_args(args):
-    '''Map existing options to format choice.'''
+    '''Map existing options to format and output choices.'''
+    # formats
     if args.csv:
         args.output_format = 'csv'
     elif args.json:
@@ -175,6 +206,28 @@ def map_args(args):
         args.output_format = 'xml'
     elif args.xmltei:
         args.output_format = 'xmltei'
+    # output configuration
+    if args.nocomments is False:
+        args.no_comments = False
+        warnings.warn(
+            """--nocomments will be deprecated in a future version,
+               use --no-comments instead""",
+             PendingDeprecationWarning
+        )
+    if args.notables is False:
+        args.no_tables = False
+        warnings.warn(
+            """--notables will be deprecated in a future version,
+               use --no-tables instead""",
+             PendingDeprecationWarning
+        )
+    if args.with_metadata is True:
+        args.only_with_metadata = True
+        warnings.warn(
+            """--with-metadata will be deprecated in a future version,
+               use --only-with-metadata instead""",
+             PendingDeprecationWarning
+        )
     return args
 
 
@@ -184,8 +237,32 @@ def main():
     process_args(args)
 
 
+def dump_on_exit(inputdict=None):
+    """Write all remaining URLs still in the input/processing list
+       to standard output before exiting."""
+    if inputdict:
+        for hostname in inputdict:
+            for urlpath in inputdict[hostname]:
+                sys.stdout.write(f'todo: {hostname}{urlpath}' + '\n')
+
+atexit.register(dump_on_exit, INPUTDICT)
+
+
+def process_parallel_results(future_to_url, blacklist, url_filter, inputdict):
+    """Process results from the parallel threads and add them
+       to the compressed URL dictionary for further processing."""
+    for future in as_completed(future_to_url):
+        if future.result() is not None:
+            inputdict = add_to_compressed_dict(
+                        future.result(), blacklist=blacklist,
+                        url_filter=url_filter, inputdict=inputdict
+                        )
+    return inputdict
+
+
 def process_args(args):
     """Perform the actual processing according to the arguments"""
+    global INPUTDICT
     # verbosity
     if args.verbose == 1:
         logging.basicConfig(stream=sys.stdout, level=logging.WARNING)
@@ -193,49 +270,67 @@ def process_args(args):
         logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)
     if args.blacklist:
         args.blacklist = load_blacklist(args.blacklist)
+
     # processing according to mutually exclusive options
     # read url list from input file
-    if args.inputfile and all([args.feed is False, args.sitemap is False, args.crawl is False]):
-        inputdict = load_input_dict(args)
-        url_processing_pipeline(args, inputdict)
+    if args.inputfile and all([args.feed is False, args.sitemap is False, args.crawl is False, args.explore is False]):
+        INPUTDICT = load_input_dict(args)
+        url_processing_pipeline(args, INPUTDICT)
+
     # fetch urls from a feed or a sitemap
-    elif args.feed or args.sitemap:
+    elif args.explore or args.feed or args.sitemap:
         input_urls = load_input_urls(args)
-        inputdict = None
         # link discovery and storage
         with ThreadPoolExecutor(max_workers=args.parallel) as executor:
             if args.feed:
                 future_to_url = {executor.submit(find_feed_urls, url, target_lang=args.target_language): url for url in input_urls}
-            elif args.sitemap:
+            elif args.explore or args.sitemap:
                 future_to_url = {executor.submit(sitemap_search, url, target_lang=args.target_language): url for url in input_urls}
-            # process results one-by-one, i.e. in parallel
-            for future in as_completed(future_to_url):
-                if future.result() is not None:
-                    inputdict = add_to_compressed_dict(future.result(), blacklist=args.blacklist, url_filter=args.url_filter, inputdict=inputdict)
+            # process results
+            with THREAD_LOCK:
+                INPUTDICT = process_parallel_results(future_to_url, args.blacklist, args.url_filter, INPUTDICT)
+                # list all links found to free memory
+                if args.list:
+                    url_processing_pipeline(args, INPUTDICT)
+
         # process the links found
-        url_processing_pipeline(args, inputdict)
+        url_processing_pipeline(args, INPUTDICT)
+
+        # activate site explorer
+        if args.explore:
+            # find domains for which nothing has been found and crawl
+            control_dict = add_to_compressed_dict(input_urls, blacklist=args.blacklist, url_filter=args.url_filter)
+            still_to_crawl = {
+                key: control_dict[key]
+                for key in control_dict
+                if key not in INPUTDICT
+            }
+            # add to compressed dict and crawl the remaining websites
+            cli_crawler(args, n=100, domain_dict=still_to_crawl)
+
     # activate crawler/spider
     elif args.crawl:
         cli_crawler(args)
+
     # read files from an input directory
     elif args.inputdir:
         file_processing_pipeline(args)
-    # read from input directly
+
+    # process input URL
+    elif args.URL:
+        INPUTDICT = add_to_compressed_dict([args.URL], args.blacklist)
+        url_processing_pipeline(args, INPUTDICT)  # process single url
+
+    # read input on STDIN directly
     else:
-        # process input URL
-        if args.URL:
-            inputdict = add_to_compressed_dict([args.URL], args.blacklist)
-            url_processing_pipeline(args, inputdict)  # process single url
-        # process input on STDIN
-        else:
-            # file type and unicode check
-            try:
-                htmlstring = sys.stdin.read()
-            except UnicodeDecodeError:
-                sys.exit('ERROR: system, file type or buffer encoding')
-            # process
-            result = examine(htmlstring, args, url=args.URL)
-            write_result(result, args)
+        # file type and unicode check
+        try:
+            htmlstring = sys.stdin.read()
+        except UnicodeDecodeError:
+            sys.exit('ERROR: system, file type or buffer encoding')
+        # process
+        result = examine(htmlstring, args, url=args.URL)
+        write_result(result, args)
 
 
 if __name__ == '__main__':
