@@ -46,7 +46,7 @@ def load_input_urls(args):
                 for line in inputfile:
                     url_match = re.match(r'https?://[^\s]+', line)
                     if url_match:
-                        input_urls.append(url_match.group(0))
+                        input_urls.append(url_match[0])
 
         except UnicodeDecodeError:
             sys.exit('ERROR: system, file type or buffer encoding')
