@@ -33,7 +33,7 @@ from .htmlprocessing import (convert_tags, handle_textnode, process_node,
                              prune_unwanted_nodes, tree_cleaning)
 from .metadata import extract_metadata, Document
 from .settings import use_config, DEFAULT_CONFIG, TAG_CATALOG
-from .utils import load_html, normalize_unicode, trim, txttocsv, is_image_file
+from .utils import is_image_file, load_html, normalize_unicode, trim, txttocsv
 from .xml import (build_json_output, build_xml_output, build_tei_output,
                   control_xml_output, xmltotxt)
 from .xpaths import (BODY_XPATH, COMMENTS_XPATH, COMMENTS_DISCARD_XPATH, OVERALL_DISCARD_XPATH,
@@ -51,6 +51,8 @@ FORMATTING = {'hi', 'ref', 'span'}
 CODES_QUOTES = {'code', 'quote'}
 NOT_AT_THE_END = {'fw', 'head', 'ref'}
 
+JSON_SEARCH = re.compile(r'"articlebody": *"(.+?)(?<!\\)"', re.I)
+
 
 def handle_titles(element, dedupbool, config):
     '''Process head elements (titles)'''
@@ -382,8 +384,7 @@ def handle_image(element):
     return processed_element
 
 
-def recover_wild_text(tree, result_body, favor_precision=False, favor_recall=False, potential_tags=TAG_CATALOG,
-                      deduplicate=True, config=None):
+def recover_wild_text(tree, result_body, favor_precision=False, favor_recall=False, potential_tags=TAG_CATALOG, deduplicate=True, config=None):
     '''Look for all previously unconsidered wild elements, including outside of the determined
        frame and throughout the document to recover potentially missing text parts'''
     LOGGER.debug('Recovering wild text elements')
@@ -398,11 +399,8 @@ def recover_wild_text(tree, result_body, favor_precision=False, favor_recall=Fal
         strip_tags(search_tree, 'a', 'ref', 'span')
     else:
         strip_tags(search_tree, 'span')
-    result_body.extend(e for e in
-                       [handle_textelem(
-                           element, potential_tags, deduplicate, config)
-                           for element in search_tree.iter(search_list)]
-                       if e is not None)
+    result_body.extend(filter(None.__ne__, (handle_textelem(e, potential_tags, deduplicate, config)
+                              for e in search_tree.iter(search_list))))
     return result_body
 
 
@@ -461,8 +459,7 @@ def prune_unwanted_sections(tree, potential_tags, favor_recall, favor_precision)
     return tree
 
 
-def extract_content(tree, favor_precision=False, favor_recall=False, include_tables=False, include_images=False,
-                    include_links=False, deduplicate=False, config=None):
+def extract_content(tree, favor_precision=False, favor_recall=False, include_tables=False, include_images=False, include_links=False, deduplicate=False, config=None):
     '''Find the main content of a page using a set of XPath expressions,
        then extract relevant elements, strip them of unwanted subparts and
        convert them'''
@@ -515,10 +512,8 @@ def extract_content(tree, favor_precision=False, favor_recall=False, include_tab
         # e.g. only lb-elems in a div
         if set(e.tag for e in subelems) == {'lb'}:
             subelems = [subtree]
-        # extract content # list(filter(None.__ne__, processed_elems)) ?
-        result_body.extend(e for e in
-                           [handle_textelem(e, potential_tags, deduplicate, config) for e in subelems]
-                           if e is not None)
+        # extract content
+        result_body.extend(filter(None.__ne__, (handle_textelem(e, potential_tags, deduplicate, config) for e in subelems)))
         # remove trailing titles
         while len(result_body) > 0 and (result_body[-1].tag in NOT_AT_THE_END):
             result_body[-1].getparent().remove(result_body[-1])
@@ -526,14 +521,14 @@ def extract_content(tree, favor_precision=False, favor_recall=False, include_tab
         if len(result_body) > 1:
             LOGGER.debug(expr)
             break
-    temp_text = trim(' '.join(result_body.itertext()))
+    temp_text = ' '.join(result_body.itertext()).strip()
     # try parsing wild <p> elements if nothing found or text too short
     # todo: test precision and recall settings here
     if len(result_body) == 0 or len(temp_text) < config.getint('DEFAULT', 'MIN_EXTRACTED_SIZE'):
         result_body = recover_wild_text(backup_tree, result_body, favor_precision=favor_precision,
                                         favor_recall=favor_recall, potential_tags=potential_tags,
                                         deduplicate=deduplicate, config=config)
-        temp_text = trim(' '.join(result_body.itertext()))
+        temp_text = ' '.join(result_body.itertext()).strip()
     # filter output
     strip_elements(result_body, 'done')
     strip_tags(result_body, 'div')
@@ -576,9 +571,9 @@ def extract_comments(tree, dedupbool, config):
         #    processed_elem = process_comments_node(elem, potential_tags)
         #    if processed_elem is not None:
         #        comments_body.append(processed_elem)
-        processed_elems = (process_comments_node(elem, potential_tags, dedupbool, config) for elem in
-                           subtree.xpath('.//*'))
-        comments_body.extend(elem for elem in processed_elems if elem is not None)
+        # processed_elems = (process_comments_node(elem, potential_tags, dedupbool, config) for elem in
+        #                    subtree.xpath('.//*'))
+        comments_body.extend(filter(None.__ne__, (process_comments_node(e, potential_tags, dedupbool, config) for e in subtree.xpath('.//*'))))
         # control
         if len(comments_body) > 0:  # if it has children
             LOGGER.debug(expr)
@@ -586,12 +581,11 @@ def extract_comments(tree, dedupbool, config):
             subtree.getparent().remove(subtree)
             break
     # lengths
-    temp_comments = trim(' '.join(comments_body.itertext()))
+    temp_comments = ' '.join(comments_body.itertext()).strip()
     return comments_body, temp_comments, len(temp_comments), tree
 
 
-def compare_extraction(tree, backup_tree, url, body, text, len_text, target_language, favor_precision, favor_recall,
-                       include_formatting, include_links, include_images, include_tables, config):
+def compare_extraction(tree, backup_tree, url, body, text, len_text, target_language, favor_precision, favor_recall, include_formatting, include_links, include_images, include_tables, config):
     '''Decide whether to choose own or external extraction
        based on a series of heuristics'''
     min_target_length = config.getint('DEFAULT', 'MIN_EXTRACTED_SIZE')
@@ -620,9 +614,9 @@ def compare_extraction(tree, backup_tree, url, body, text, len_text, target_lang
         algo_flag = True
     # borderline cases
     else:
-        if not body.xpath('//p//text()') and len_algo > min_target_length * 2:
+        if not body.xpath('.//p//text()') and len_algo > min_target_length * 2:
             algo_flag = True
-        elif len(body.xpath('//table')) > len(body.xpath('//p')) and len_algo > min_target_length * 2:
+        elif len(body.findall('.//table')) > len(body.findall('.//p')) and len_algo > min_target_length * 2:
             algo_flag = True
         else:
             LOGGER.debug('extraction values: %s %s for %s', len_text, len_algo, url)
@@ -634,7 +628,7 @@ def compare_extraction(tree, backup_tree, url, body, text, len_text, target_lang
     else:
         LOGGER.info('using custom extraction: %s', url)
     # override faulty extraction: try with justext
-    if body.xpath(SANITIZED_XPATH) or len_text < min_target_length:
+    if body.xpath(SANITIZED_XPATH) or len_text < min_target_length:  # body.find(...)
     # or favor_recall is True ?
         # tree = prune_unwanted_sections(tree, {}, favor_recall, favor_precision)
         body2, text2, len_text2, jt_result = justext_rescue(tree, url, target_language, body, 0, '')
@@ -665,19 +659,19 @@ def baseline(filecontent):
     # scrape from json text
     for elem in tree.iterfind('.//script[@type="application/ld+json"]'):
         if elem.text and '"article' in elem.text:
-            mymatch = re.search(r'"articlebody": *"(.+?)(?<!\\)"', elem.text, re.I)
+            mymatch = JSON_SEARCH.search(elem.text)
             if mymatch:
                 elem = SubElement(postbody, 'p')
-                elem.text = trim(mymatch.group(1).replace('\\"', '"'))
+                elem.text = trim(mymatch[1].replace('\\"', '"'))
                 return postbody, elem.text, len(elem.text)
     # basic tree cleaning
-    for elem in tree.xpath('//aside|//footer|//script|//style'):
+    for elem in tree.xpath('.//aside|.//footer|.//script|.//style'):
         elem.getparent().remove(elem)
     # scrape from article tag
     article_elem = tree.find('.//article')
     if article_elem is not None:
         temp_text = trim(article_elem.text_content())
-        if len(temp_text) > 0:
+        if len(temp_text) > 100:
             elem = SubElement(postbody, 'p')
             elem.text = temp_text
             return postbody, temp_text, len(temp_text)
@@ -690,17 +684,38 @@ def baseline(filecontent):
             elem.text = entry
             results.add(entry)
     temp_text = trim('\n'.join(postbody.itertext()))
-    if len(temp_text) > 0:
+    if len(temp_text) > 100:
         return postbody, temp_text, len(temp_text)
     # default strategy: clean the tree and take everything
     postbody = Element('body')
     body_elem = tree.find('.//body')
     if body_elem is not None:
-        elem = SubElement(postbody, 'p')
         # elem.text = trim(body_elem.text_content())
-        elem.text = '\n'.join([trim(e) for e in body_elem.itertext()])
-        return postbody, elem.text, len(elem.text)
-    return postbody, '', 0
+        text = '\n'.join([trim(e) for e in body_elem.itertext()])
+        if len(text) > 100:
+            elem = SubElement(postbody, 'p')
+            elem.text = text
+            return postbody, text, len(text)
+    # new fallback
+    text = html2txt(tree)
+    elem = SubElement(postbody, 'p')
+    elem.text = text
+    return postbody, text, len(text)
+    # old: return postbody, '', 0
+
+
+def html2txt(content):
+    """Run basic html2txt on a document.
+
+    Args:
+        content: HTML document as string or LXML element.
+
+    Returns:
+        The extracted text in the form of a string.
+
+    """
+    tree = load_html(content)
+    return ' '.join(tree.text_content().split()).strip()
 
 
 def determine_returnstring(document, output_format, include_formatting, tei_validation):
@@ -838,23 +853,18 @@ def bare_extraction(filecontent, url=None, no_fallback=False,
 
         # comments first, then remove
         if include_comments is True:
-            commentsbody, temp_comments, len_comments, cleaned_tree = extract_comments(cleaned_tree, deduplicate,
-                                                                                       config)
+            commentsbody, temp_comments, len_comments, cleaned_tree = extract_comments(cleaned_tree, deduplicate, config)
         else:
             commentsbody, temp_comments, len_comments = None, '', 0
         if favor_precision is True:
             cleaned_tree = prune_unwanted_nodes(cleaned_tree, REMOVE_COMMENTS_XPATH)
 
         # extract content
-        postbody, temp_text, len_text = extract_content(cleaned_tree, favor_precision, favor_recall, include_tables,
-                                                        include_images, include_links, deduplicate, config)
+        postbody, temp_text, len_text = extract_content(cleaned_tree, favor_precision, favor_recall, include_tables, include_images, include_links, deduplicate, config)
 
         # compare if necessary
         if no_fallback is False:
-            postbody, temp_text, len_text = compare_extraction(cleaned_tree_backup, tree_backup_1, url, postbody,
-                                                               temp_text, len_text, target_language, favor_precision,
-                                                               favor_recall, include_formatting, include_links,
-                                                               include_images, include_tables, config)
+            postbody, temp_text, len_text = compare_extraction(cleaned_tree_backup, tree_backup_1, url, postbody, temp_text, len_text, target_language, favor_precision, favor_recall, include_formatting, include_links, include_images, include_tables, config)
         # add baseline as additional fallback
         # rescue: try to use original/dirty tree # and favor_precision is False=?
         if len_text < config.getint('DEFAULT', 'MIN_EXTRACTED_SIZE'):
@@ -1015,4 +1025,15 @@ def extract(filecontent, url=None, record_id=None, no_fallback=False,
 
 
 # for legacy and backwards compatibility
-process_record = extract
+def process_record(filecontent, url=None, record_id=None, no_fallback=False,
+                   include_comments=True, target_language=None,
+                   include_tables=True):
+    "Legacy extraction function, now deprecated."
+    # deprecation warning
+    warnings.warn(
+        "process_record() is deprecated, use extract() instead",
+        DeprecationWarning
+    )
+    return extract(filecontent, url=url, record_id=record_id, no_fallback=no_fallback,
+                   include_comments=include_comments, target_language=target_language,
+                   include_tables=include_tables)
