@@ -7,8 +7,8 @@ Functions to process nodes in HTML code.
 ## under GNU GPL v3 license
 
 import logging
-import re
 
+from collections import defaultdict
 from copy import deepcopy
 
 from lxml.etree import strip_tags
@@ -111,30 +111,28 @@ def prune_unwanted_nodes(tree, nodelist, with_backup=False):
 
 def collect_link_info(links_xpath, favor_precision=False):
     '''Collect heuristics on link text'''
-    linklen, elemnum, shortelems, mylist = 0, 0, 0, []
+    # init
+    shortelems, mylist = 0, []
+    # longer strings impact recall in favor of precision
+    if favor_precision is False:
+        threshold = 10
+    else:
+        threshold = 50
+    # examine the elements
     for subelem in links_xpath:
         subelemtext = trim(subelem.text_content())
-        subelemlen = len(subelemtext)
-        if subelemlen == 0:
+        if not subelemtext:
             continue
-        linklen += subelemlen
-        elemnum += 1
-        # longer strings impact recall in favor of precision
-        if favor_precision is False:
-            threshold = 10
-        else:
-            threshold = 50
-        if subelemlen < threshold:
-            shortelems += 1
         mylist.append(subelemtext)
-    return linklen, elemnum, shortelems, mylist
+    lengths = [len(text) for text in mylist]
+    shortelems = len([l for l in lengths if l < threshold])
+    return sum(lengths), len(mylist), shortelems, mylist
 
 
-def link_density_test(element, favor_precision=False):
+def link_density_test(element, text, favor_precision=False):
     '''Remove sections which are rich in links (probably boilerplate)'''
-    links_xpath, mylist = element.xpath('.//ref'), []
+    links_xpath, mylist = element.findall('.//ref'), []
     if links_xpath:
-        elemlen = len(trim(element.text_content()))
         if element.tag == 'p': #  and not element.getparent().tag == 'item'
             #if element.getnext() is None:
             #    limitlen, threshold = 100, 0.8
@@ -154,6 +152,7 @@ def link_density_test(element, favor_precision=False):
             #    limitlen, threshold = 150, 0.66
             else:
                 limitlen, threshold = 100, 0.8
+        elemlen = len(text)
         if elemlen < limitlen:
             linklen, elemnum, shortelems, mylist = collect_link_info(links_xpath, favor_precision)
             if elemnum == 0:
@@ -168,8 +167,8 @@ def link_density_test(element, favor_precision=False):
 def link_density_test_tables(element):
     '''Remove tables which are rich in links (probably boilerplate)'''
     # if element.getnext() is not None:
-    #    return False
-    links_xpath = element.xpath('.//ref')
+    #     return False
+    links_xpath = element.findall('.//ref')
     if links_xpath:
         elemlen = len(trim(element.text_content()))
         if elemlen > 250:
@@ -188,17 +187,14 @@ def link_density_test_tables(element):
 def delete_by_link_density(subtree, tagname, backtracking=False, favor_precision=False):
     '''Determine the link density of elements with respect to their length,
        and remove the elements identified as boilerplate.'''
-    myelems, deletions = {}, []
+    myelems, deletions = defaultdict(list), []
     for elem in subtree.iter(tagname):
-        result, templist = link_density_test(elem, favor_precision)
+        elemtext = trim(elem.text_content())
+        result, templist = link_density_test(elem, elemtext, favor_precision)
         if result is True:
             deletions.append(elem)
-        elif backtracking is True and len(templist) > 0:
-            text = trim(elem.text_content())
-            if text not in myelems:
-                myelems[text] = [elem]
-            else:
-                myelems[text].append(elem)
+        elif backtracking is True and len(templist) > 0:  # if?
+            myelems[elemtext].append(elem)
     # summing up
     if backtracking is True:
         if favor_precision is False:
@@ -221,21 +217,12 @@ def delete_by_link_density(subtree, tagname, backtracking=False, favor_precision
 
 def convert_tags(tree, include_formatting=False, include_tables=False, include_images=False, include_links=False):
     '''Simplify markup and convert relevant HTML tags to an XML standard'''
-    # ul/ol → list / li → item
-    for elem in tree.iter('ul', 'ol', 'dl'):
-        elem.tag = 'list'
-        for subelem in elem.iter('dd', 'dt', 'li'):
-            subelem.tag = 'item'
-    # images
-    if include_images is True:
-        for elem in tree.iter('img'):
-            elem.tag = 'graphic'
     # delete links for faster processing
     if include_links is False:
         if include_tables is True:
-            xpath_expr = '//div//a|//list//a|//table//a'
+            xpath_expr = './/div//a|.//table//a|.//ul//a'  # .//p//a ?
         else:
-            xpath_expr = '//div//a|//list//a'
+            xpath_expr = './/div//a|.//ul//a'  # .//p//a ?
         # necessary for further detection
         for elem in tree.xpath(xpath_expr):
             elem.tag = 'ref'
@@ -249,54 +236,66 @@ def convert_tags(tree, include_formatting=False, include_tables=False, include_i
             elem.attrib.clear()
             if target is not None:
                 elem.set('target', target)
-    # head tags + delete attributes
-    for elem in tree.iter('h1', 'h2', 'h3', 'h4', 'h5', 'h6'):
-        elem.attrib.clear()
-        elem.set('rend', elem.tag)
-        elem.tag = 'head'
-    # br → lb
-    for elem in tree.iter('br', 'hr'):
-        elem.tag = 'lb'
-    # wbr
-    # blockquote, pre, q → quote
-    for elem in tree.iter('blockquote', 'pre', 'q'):
-        elem.tag = 'quote'
     # include_formatting
     if include_formatting is False:
         strip_tags(tree, 'em', 'i', 'b', 'strong', 'u', 'kbd', 'samp', 'tt', 'var', 'sub', 'sup')
     else:
-        # italics
-        for elem in tree.iter('em', 'i'):
-            elem.tag = 'hi'
-            elem.set('rend', '#i')
-        # bold font
-        for elem in tree.iter('b', 'strong'):
-            elem.tag = 'hi'
-            elem.set('rend', '#b')
-        # u (very rare)
-        for elem in tree.iter('u'):
-            elem.tag = 'hi'
-            elem.set('rend', '#u')
-        # tt (very rare)
-        for elem in tree.iter('kbd', 'samp', 'tt', 'var'):
-            elem.tag = 'hi'
-            elem.set('rend', '#t')
-        # sub and sup (very rare)
-        for elem in tree.iter('sub'):
-            elem.tag = 'hi'
-            elem.set('rend', '#sub')
-        for elem in tree.iter('sup'):
-            elem.tag = 'hi'
-            elem.set('rend', '#sup')
-    # del | s | strike → <del rend="overstrike">
-    for elem in tree.iter('del', 's', 'strike'):
-        elem.tag = 'del'
-        elem.set('rend', 'overstrike')
-    # details + summary
-    for elem in tree.iter('details'):
-        elem.tag = 'div'
-        for subelem in elem.iter('summary'):
-            subelem.tag = 'head'
+        for elem in tree.iter('em', 'i', 'b', 'strong', 'u', 'kbd', 'samp', 'tt', 'var', 'sub', 'sup'):
+            # italics
+            if elem.tag in ('em', 'i'):
+                elem.tag = 'hi'
+                elem.set('rend', '#i')
+            # bold font
+            elif elem.tag in ('b', 'strong'):
+                elem.tag = 'hi'
+                elem.set('rend', '#b')
+            # u (very rare)
+            elif elem.tag == 'u':
+                elem.tag = 'hi'
+                elem.set('rend', '#u')
+            # tt (very rare)
+            elif elem.tag in ('kbd', 'samp', 'tt', 'var'):
+                elem.tag = 'hi'
+                elem.set('rend', '#t')
+            # sub and sup (very rare)
+            elif elem.tag == 'sub':
+                elem.tag = 'hi'
+                elem.set('rend', '#sub')
+            elif elem.tag == 'sup':
+                elem.tag = 'hi'
+                elem.set('rend', '#sup')
+    # iterate over all concerned elements
+    for elem in tree.iter('blockquote', 'br', 'del', 'details', 'dl', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'hr', 'ol', 'pre', 'q', 's', 'strike', 'ul'):
+        # ul/ol → list / li → item
+        if elem.tag in ('dl', 'ol', 'ul'):
+            elem.tag = 'list'
+            for subelem in elem.iter('dd', 'dt', 'li'):
+                subelem.tag = 'item'
+        # head tags + delete attributes
+        elif elem.tag in ('h1', 'h2', 'h3', 'h4', 'h5', 'h6'):
+            elem.attrib.clear()
+            elem.set('rend', elem.tag)
+            elem.tag = 'head'
+        # br → lb
+        elif elem.tag in ('br', 'hr'):
+            elem.tag = 'lb'
+        # wbr
+        # blockquote, pre, q → quote
+        elif elem.tag in ('blockquote', 'pre', 'q'):
+            elem.tag = 'quote'
+        # del | s | strike → <del rend="overstrike">
+        elif elem.tag in ('del', 's', 'strike'):
+            elem.tag = 'del'
+            elem.set('rend', 'overstrike')
+        # details + summary
+        elif elem.tag == 'details':
+            elem.tag = 'div'
+            for subelem in elem.iter('summary'):
+                subelem.tag = 'head'
+    # images
+    if include_images is True:
+        for elem in tree.iter('img'):
+            elem.tag = 'graphic'
     return tree
 
 
@@ -325,7 +324,7 @@ def handle_textnode(element, comments_fix=True, deduplicate=True, preserve_space
         if element.tail:
             element.tail = trim(element.tail)
     # filter content
-    if not element.text or not re.search(r'\w', element.text):  # text_content()?
+    if not element.text:  # or not re.search(r'\w', element.text):  # text_content()?
         return None
     if textfilter(element) is True:
         return None
