@@ -56,7 +56,7 @@ HTMLDATE_CONFIG_EXTENSIVE = {'extensive_search': True, 'original_date': True}
 JSON_MINIFY = re.compile(r'("(?:\\"|[^"])*")|\s')
 
 HTMLTITLE_REGEX = re.compile(r'^(.+)?\s+[-|]\s+(.+)$')  # part without dots?
-URL_COMP_CHECK = re.compile(r'https?://|/')
+URL_COMP_CHECK = re.compile(r'https?://')
 HTML_STRIP_TAG = re.compile(r'(<!--.*?-->|<[^>]*>)')
 
 LICENSE_REGEX = re.compile(r'/(by-nc-nd|by-nc-sa|by-nc|by-nd|by-sa|by|zero)/([1-9]\.[0-9])')
@@ -243,11 +243,11 @@ def examine_title_element(tree):
     '''Extract text segments out of main <title> element.'''
     title, first, second = None, None, None
     try:
-        title = trim(tree.xpath('//head//title')[0].text_content())
+        title = trim(tree.xpath('.//head//title')[0].text_content())
         mymatch = HTMLTITLE_REGEX.match(title)
         if mymatch is not None:
-            first = mymatch.group(1) or None
-            second = mymatch.group(2) or None
+            first = mymatch[1] or None
+            second = mymatch[2] or None
     except IndexError:
         LOGGER.warning('no main title found')
     return title, first, second
@@ -256,7 +256,7 @@ def examine_title_element(tree):
 def extract_title(tree):
     '''Extract the document title'''
     # only one h1-element: take it
-    h1_results = tree.xpath('//h1')
+    h1_results = tree.findall('.//h1')
     if len(h1_results) == 1:
         title = trim(h1_results[0].text_content())
         if len(title) > 0:
@@ -276,7 +276,7 @@ def extract_title(tree):
         return h1_results[0].text_content()
     # take first h2-title
     try:
-        title = tree.xpath('//h2')[0].text_content()
+        title = tree.xpath('.//h2')[0].text_content()
     except IndexError:
         LOGGER.warning('no h2 title found')
     return title
@@ -323,7 +323,7 @@ def extract_url(tree, default_url=None):
                 domain_match = re.match(r'https?://[^/]+', element.attrib['content'])
                 if domain_match:
                     # prepend URL
-                    url = domain_match.group(0) + url
+                    url = domain_match[0] + url
                     break
     # sanity check: don't return invalid URLs
     if url is not None:
@@ -374,7 +374,7 @@ def parse_license_element(element, strict=False):
    # look for Creative Commons elements
     match = LICENSE_REGEX.search(element.get('href'))
     if match:
-        return 'CC ' + match.group(1).upper() + ' ' + match.group(2)
+        return 'CC ' + match[1].upper() + ' ' + match[2]
     if element.text is not None:
         # just return the anchor text without further ado
         if strict is False:
@@ -382,7 +382,7 @@ def parse_license_element(element, strict=False):
         # else: check if it could be a CC license
         match = TEXT_LICENSE_REGEX.search(element.text)
         if match:
-            return match.group(0)
+            return match[0]
     return None
 
 
@@ -390,14 +390,14 @@ def extract_license(tree):
     '''Search the HTML code for license information and parse it.'''
     result = None
     # look for links labeled as license
-    for element in tree.xpath('//a[@rel="license"][@href]'):
+    for element in tree.findall('.//a[@rel="license"][@href]'):
         result = parse_license_element(element, strict=False)
         if result is not None:
             break
     # probe footer elements for CC links
     if result is None:
         for element in tree.xpath(
-            '//footer//a[@href]|//div[contains(@class, "footer") or contains(@id, "footer")]//a[@href]'
+            './/footer//a[@href]|.//div[contains(@class, "footer") or contains(@id, "footer")]//a[@href]'
         ):
             result = parse_license_element(element, strict=True)
             if result is not None:
@@ -468,6 +468,9 @@ def extract_metadata(filecontent, default_url=None, date_config=None, fastmode=F
     if metadata.sitename is None:
         metadata.sitename = extract_sitename(tree)
     if metadata.sitename is not None:
+        # fix: take 1st element (['Westdeutscher Rundfunk'])
+        if isinstance(metadata.sitename, list):
+            metadata.sitename = metadata.sitename[0]
         if metadata.sitename.startswith('@'):
             # scrap Twitter ID
             metadata.sitename = re.sub(r'^@', '', metadata.sitename)
@@ -485,7 +488,7 @@ def extract_metadata(filecontent, default_url=None, date_config=None, fastmode=F
     elif metadata.url:
         mymatch = re.match(r'https?://(?:www\.|w[0-9]+\.)?([^/]+)', metadata.url)
         if mymatch:
-            metadata.sitename = mymatch.group(1)
+            metadata.sitename = mymatch[1]
     # categories
     if not metadata.categories:
         metadata.categories = extract_catstags('category', tree)
